package factoreasyrest.facade.impl;


import eu.cz.culs.kii.common.factoreasycommon.dbentities.FactoriesGameSettings;
import factoreasyrest.facade.FactOrEasyGameSettingsFacade;
import factoreasyrest.model.entities.FactOrEasyGameSettingsModel;
import factoreasyrest.service.FactOrEasyGameSettingsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Component
public class FactOrEasyGameSettingsFacadeImpl implements FactOrEasyGameSettingsFacade {

    @Autowired
    FactOrEasyGameSettingsService factOrEasyGameSettingsService;

    @Override
    public FactOrEasyGameSettingsModel read(Long id) {
        FactoriesGameSettings factoriesGameSettings = factOrEasyGameSettingsService.read(id);
        FactOrEasyGameSettingsModel model = convertEntityToModel(factoriesGameSettings);
        return model;
    }

    @Override
    public FactOrEasyGameSettingsModel create(FactOrEasyGameSettingsModel gameSettings) {
        FactoriesGameSettings factoriesGameSettings = convertModelToEntity(gameSettings);
        factoriesGameSettings = factOrEasyGameSettingsService.create(factoriesGameSettings);
        return convertEntityToModel(factoriesGameSettings);
    }

    @Override
    public FactOrEasyGameSettingsModel modify(FactOrEasyGameSettingsModel gameSettings, Long id) {
        FactoriesGameSettings factoriesGameSettings = factOrEasyGameSettingsService.read(id);
        FactOrEasyGameSettingsModel model;
        if(Objects.nonNull(factoriesGameSettings)){
            model = convertEntityToModel(factOrEasyGameSettingsService.modify(convertModelToEntity(gameSettings)));
        } else {
            factoriesGameSettings =  factOrEasyGameSettingsService.create(convertModelToEntity(gameSettings));
            model = convertEntityToModel(factoriesGameSettings);
        }
        return model;
    }

    @Override
    public void delete(Long id) {
        factOrEasyGameSettingsService.delete(id);
    }

    @Override
    public List<FactOrEasyGameSettingsModel> readAll() {
        List<FactOrEasyGameSettingsModel> factOrEasyGameSettingsModelList =
                convertEntityListToModelList(factOrEasyGameSettingsService.readAll());
        return factOrEasyGameSettingsModelList;
    }

    private List<FactOrEasyGameSettingsModel> convertEntityListToModelList(List<FactoriesGameSettings> factoriesGameSettingsList) {
        List<FactOrEasyGameSettingsModel> factOrEasyGameSettingsModelList = new ArrayList<FactOrEasyGameSettingsModel>();
        for(FactoriesGameSettings factoriesGameSettings : factoriesGameSettingsList) {
            factOrEasyGameSettingsModelList.add(convertEntityToModel(factoriesGameSettings));
        }
        return factOrEasyGameSettingsModelList;
    }

    private FactoriesGameSettings convertModelToEntity(FactOrEasyGameSettingsModel factOrEasyGameSettingsModel){
        FactoriesGameSettings factoriesGameSettings = new FactoriesGameSettings();
        factoriesGameSettings.setId(factOrEasyGameSettingsModel.getId());
        factoriesGameSettings.setAccount(factOrEasyGameSettingsModel.getAccount());
        factoriesGameSettings.setNumberOfPlayers(factOrEasyGameSettingsModel.getNumberOfPlayers());
        factoriesGameSettings.setProductsStoreCosts(factOrEasyGameSettingsModel.getProductsStoreCosts());
        factoriesGameSettings.setStandardFactoryFixedCost(factOrEasyGameSettingsModel.getStandardFactoryFixedCost());
        factoriesGameSettings.setAutomatizedFactoryConstructionCosts(factOrEasyGameSettingsModel.getAutomatizedFactoryConstructionCosts());
        factoriesGameSettings.setMaterialStore(factOrEasyGameSettingsModel.getMaterialStore());
        factoriesGameSettings.setProductInStore(factOrEasyGameSettingsModel.getProductInStore());
        factoriesGameSettings.setStandardDevelopmentCost(factOrEasyGameSettingsModel.getStandardDevelopmentCost());
        factoriesGameSettings.setAutomatizedDevelopmentCosts(factOrEasyGameSettingsModel.getAutomatizedDevelopmentCosts());
        factoriesGameSettings.setStandardFactoryNumber(factOrEasyGameSettingsModel.getStandardFactoryNumber());
        factoriesGameSettings.setAutomatizedFactoryNumber(factOrEasyGameSettingsModel.getAutomatizedFactoryNumber());
        factoriesGameSettings.setInitialNumberOfGamesPerPlayer(factOrEasyGameSettingsModel.getInitialNumberOfGamesPerPlayer());
        factoriesGameSettings.setLapsPerSimulation(factOrEasyGameSettingsModel.getLapsPerSimulation());
        factoriesGameSettings.setLoanLimitRatio(factOrEasyGameSettingsModel.getLoanLimitRatio());
        factoriesGameSettings.setBotBuyFactoryAggressionLevel(factOrEasyGameSettingsModel.getBotBuyFactoryAggressionLevel());
        return factoriesGameSettings;
    }

    private FactOrEasyGameSettingsModel convertEntityToModel(FactoriesGameSettings factoriesGameSettings){
        FactOrEasyGameSettingsModel factOrEasyGameSettingsModel = new FactOrEasyGameSettingsModel(
                factoriesGameSettings.getId(),
                factoriesGameSettings.getAccount(),
                factoriesGameSettings.getNumberOfPlayers(),
                factoriesGameSettings.getMaterialStoreCosts(),
                factoriesGameSettings.getProductsStoreCosts(),
                factoriesGameSettings.getStandardFactoryFixedCost(),
                factoriesGameSettings.getAutomatizedFactoryConstructionCosts(),
                factoriesGameSettings.getMaterialStore(),
                factoriesGameSettings.getProductInStore(),
                factoriesGameSettings.getStandardDevelopmentCost(),
                factoriesGameSettings.getAutomatizedDevelopmentCosts(),
                factoriesGameSettings.getStandardFactoryNumber(),
                factoriesGameSettings.getAutomatizedFactoryNumber(),
                factoriesGameSettings.getInitialNumberOfGamesPerPlayer(),
                factoriesGameSettings.getLapsPerSimulation(),
                factoriesGameSettings.getLoanLimitRatio(),
                factoriesGameSettings.getBotBuyFactoryAggressionLevel()
        );
        return factOrEasyGameSettingsModel;
    }
}
